/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.datamodel;

import jalview.api.DBRefEntryI;

public class DBRefEntry implements DBRefEntryI
{
  String source = "", version = "", accessionId = "";

  private int startRes, endRes;
  /**
   * maps from associated sequence to the database sequence's coordinate system
   */
  Mapping map = null;

  public DBRefEntry()
  {

  }

  public DBRefEntry(String source, String version, String accessionId)
  {
    this(source, version, accessionId, null);
  }

  /**
   * 
   * @param source
   *          canonical source (uppercase only)
   * @param version
   *          (source dependent version string)
   * @param accessionId
   *          (source dependent accession number string)
   * @param map
   *          (mapping from local sequence numbering to source accession
   *          numbering)
   */
  public DBRefEntry(String source, String version, String accessionId,
          Mapping map)
  {
    this.source = source.toUpperCase();
    this.version = version;
    this.accessionId = accessionId;
    this.map = map;
  }

  public DBRefEntry(DBRefEntry entry)
  {
    this(
            (entry.source == null ? "" : new String(entry.source)),
            (entry.version == null ? "" : new String(entry.version)),
            (entry.accessionId == null ? "" : new String(entry.accessionId)),
            (entry.map == null ? null : new Mapping(entry.map)));
  }

  @Override
  public boolean equals(Object o)
  {
    // TODO should also override hashCode to ensure equal objects have equal
    // hashcodes
    if (o == null || !(o instanceof DBRefEntry))
    {
      return false;
    }
    DBRefEntry entry = (DBRefEntry) o;
    if (entry == this)
    {
      return true;
    }
    if (equalRef(entry)
            && ((map == null && entry.map == null) || (map != null
                    && entry.map != null && map.equals(entry.map))))
    {
      return true;
    }
    return false;
  }

  /**
   * test for similar DBRef attributes, except for the map object.
   * 
   * @param entry
   * @return true if source, accession and version are equal with those of entry
   */
  @Override
  public boolean equalRef(DBRefEntryI entry)
  {
    if (entry == null)
    {
      return false;
    }
    if (entry == this)
    {
      return true;
    }
    if (entry != null
            && (source != null && entry.getSource() != null && source
                    .equalsIgnoreCase(entry.getSource()))
            && (accessionId != null && entry.getAccessionId() != null && accessionId
                    .equalsIgnoreCase(entry.getAccessionId()))
            && (version != null && entry.getVersion() != null && version
                    .equalsIgnoreCase(entry.getVersion())))
    {
      return true;
    }
    return false;
  }

  @Override
  public String getSource()
  {
    return source;
  }

  @Override
  public String getVersion()
  {
    return version;
  }

  @Override
  public String getAccessionId()
  {
    return accessionId;
  }


  @Override
  public void setAccessionId(String accessionId)
  {
    this.accessionId = accessionId;
  }


  @Override
  public void setSource(String source)
  {
    this.source = source;
  }


  @Override
  public void setVersion(String version)
  {
    this.version = version;
  }


  public Mapping getMap()
  {
    return map;
  }

  /**
   * @param map
   *          the map to set
   */
  public void setMap(Mapping map)
  {
    this.map = map;
  }

  public boolean hasMap()
  {
    return map != null;
  }

  /**
   * 
   * @return source+":"+accessionId
   */
  public String getSrcAccString()
  {
    return ((source != null) ? source : "") + ":"
            + ((accessionId != null) ? accessionId : "");
  }

  @Override
  public String toString()
  {
    return getSrcAccString();
  }

  @Override
  public int getStartRes()
  {
    return startRes;
  }

  @Override
  public void setStartRes(int startRes)
  {
    this.startRes = startRes;
  }

  @Override
  public int getEndRes()
  {
    return endRes;
  }

  @Override
  public void setEndRes(int endRes)
  {
    this.endRes = endRes;
  }
}
